@@ -13,7 +13,7 @@
   Fax +61 3 9886 6616.
 */
 
-/* $Id: cdoc.c,v 1.16 2004/01/16 12:03:06 ghostgum Exp $ */
+/* $Id: cdoc.c,v 1.17 2004/11/17 02:30:15 ghostgum Exp $ */
 /* Common routines for Doc object */
 
 #include "common.h"
@@ -574,7 +574,7 @@ static PclType doc_pjl_parse(const char *str, int len, unsigned int *resolution)
 		while ((i<count) && (p[i] == ' '))
 		    i++;
 		/* now read number */
-		bufcnt = min(i-count, (int)sizeof(buf)-1);
+		bufcnt = min(count-i, (int)sizeof(buf)-1);
 		memcpy(buf, p+i, bufcnt);
 		buf[bufcnt] = '\0';
 		*resolution = atoi(buf);
