@@ -1,4 +1,4 @@
-/* Copyright (C) 1993-2004 Ghostgum Software Pty Ltd.  All rights reserved.
+/* Copyright (C) 1993-2005 Ghostgum Software Pty Ltd.  All rights reserved.
 
   This software is provided AS-IS with no warranty, either express or
   implied.
@@ -13,7 +13,7 @@
   Fax +61 3 9886 6616.
 */
 
-/* $Id: ceps.c,v 1.29 2004/01/16 08:55:36 ghostgum Exp $ */
+/* $Id: ceps.c,v 1.36 2005/01/07 11:27:37 ghostgum Exp $ */
 
 /* EPS preview manipulation */
 
@@ -50,11 +50,11 @@ static FILE_POS write_platefile_comments(Doc *doc, GFile *docfile,
     GFile *epsfile, LPCTSTR epsname,
     int offset, FILE_POS file_offset, 
     BOOL dcs2_multi, BOOL write_all, BOOL missing_separations,
-    RENAME_SEPARATION *rs);
+    const char *renamed[], BOOL some_renamed);
 static FILE_POS write_singlefile_separations(Doc *doc, GFile *docfile, 
     GFile *epsfile, 
     BOOL dcs2_multi, BOOL write_all, BOOL missing_separations,
-    RENAME_SEPARATION *rs);
+    BOOL some_renamed);
 
 /* A placeable Windows Metafile header is needed
  * when the metafile is in a WMF file, but not
@@ -605,6 +605,8 @@ CDSC *dsc = doc->dsc;
 
 static char hex[17] = "0123456789ABCDEF";
 
+#define MAXHEXWIDTH 70
+
 /* Write interchange preview to file f */
 /* Does not copy the file itself */
 int
@@ -619,22 +621,41 @@ write_interchange(GFile *f, IMAGE *img, CDSCBBOX devbbox)
     int preview_width, bwidth;
     int lines_per_scan;
     int topfirst = ((img->format & DISPLAY_FIRSTROW_MASK) == DISPLAY_TOPFIRST);
-    char hexline[90];
+    char hexline[MAXHEXWIDTH+6];
     int hexcount = 0;
+    unsigned int value;
+    unsigned int depth = 8;
     
     validate_devbbox(img, &devbbox);
 
-    /* byte width of interchange preview with 1 bit/pixel */
-    bwidth = (((devbbox.urx-devbbox.llx) + 7) & ~7) >> 3;
-    /* byte width of intermediate preview with 1 bit/pixel */
-    preview_width = ((img->width + 7) & ~7) >> 3;
+    switch (img->format & DISPLAY_COLORS_MASK) {
+	case DISPLAY_COLORS_NATIVE:
+	case DISPLAY_COLORS_GRAY:
+	    if ((img->format & DISPLAY_DEPTH_MASK) == DISPLAY_DEPTH_1)
+		depth = 1;
+    }
+
+    switch (depth) {
+	case 1:
+	    /* byte width of interchange preview with 1 bit/pixel */
+	    bwidth = (((devbbox.urx-devbbox.llx) + 7) & ~7) >> 3;
+	    /* byte width of intermediate preview with 1 bit/pixel */
+	    preview_width = ((img->width + 7) & ~7) >> 3;
+	    break;
+	case 8:
+	    bwidth = devbbox.urx-devbbox.llx;
+	    preview_width = img->width;
+	    break;
+	default:
+	    return -1;
+    }
 
     preview = (unsigned char *) malloc(preview_width);
 
-    lines_per_scan = ((bwidth-1) / 32) + 1;
+    lines_per_scan = (bwidth + (MAXHEXWIDTH/2) - 1) / (MAXHEXWIDTH/2);
     buf[sizeof(buf)-1] = '\0';
-    snprintf(buf, MAXSTR-1, "%%%%BeginPreview: %u %u 1 %u%s",
-	(devbbox.urx-devbbox.llx), (devbbox.ury-devbbox.lly), 
+    snprintf(buf, MAXSTR-1, "%%%%BeginPreview: %u %u %u %u%s",
+	(devbbox.urx-devbbox.llx), (devbbox.ury-devbbox.lly), depth,
 	(devbbox.ury-devbbox.lly)*lines_per_scan, eol_str);
     gfile_puts(f, buf);
 
@@ -646,14 +667,21 @@ write_interchange(GFile *f, IMAGE *img, CDSCBBOX devbbox)
     /* process each line of bitmap */
     for (i = 0; i < (devbbox.ury-devbbox.lly); i++) {
 	memset(preview,0xff,preview_width);
-	image_to_mono(img, preview, line);
-	if (devbbox.llx)
-	    shift_preview(preview, preview_width, devbbox.llx);
+	if (depth == 1) {
+	    image_to_mono(img, preview, line);
+	    if (devbbox.llx)
+		shift_preview(preview, preview_width, devbbox.llx);
+	}
+	else {
+	    image_to_grey(img, preview, line);
+	    if (devbbox.llx)
+		memmove(preview, preview+devbbox.llx, preview_width);
+	}
 	hexcount = 0;
 	hexline[hexcount++] = '%';
 	hexline[hexcount++] = ' ';
 	for (j=0; j<bwidth; j++) {
-	    if (hexcount >= 72) {
+	    if (hexcount >= 2 + MAXHEXWIDTH) {
 		if (strlen(eol_str) <= sizeof(hexline)-hexcount) {
 		    memcpy(hexline+hexcount, eol_str, strlen(eol_str));
 		    hexcount += strlen(eol_str);
@@ -663,8 +691,11 @@ write_interchange(GFile *f, IMAGE *img, CDSCBBOX devbbox)
 		hexline[hexcount++] = '%';
 		hexline[hexcount++] = ' ';
 	    }
-	    hexline[hexcount++] = hex[15-((preview[j]>>4)&15)];
-	    hexline[hexcount++] = hex[15-((preview[j])&15)];
+	    value = preview[j];
+	    if (depth == 8)
+		value = 255 - value;
+	    hexline[hexcount++] = hex[(value>>4)&15];
+	    hexline[hexcount++] = hex[value&15];
 	}
 	if (hexcount && (strlen(eol_str) <= sizeof(hexline)-hexcount)) {
 	    memcpy(hexline+hexcount, eol_str, strlen(eol_str));
@@ -1226,6 +1257,48 @@ separation_name(RENAME_SEPARATION *rs, const char *name)
     return newname;
 }
 
+/*
+ * Set renamed to the names of the renamed separations.
+ * renamed[] must be as long as dsc->page_count.
+ * Return 0 on success, or -1 if a separation name appears twice.
+ */
+int
+rename_separations(CDSC *dsc, RENAME_SEPARATION *rs, const char *renamed[])
+{
+    int duplicated = 0;
+    int i, j;
+    const char *sepname;
+    renamed[0] = dsc->page[0].label;
+    for (i=1; i<(int)dsc->page_count; i++) {
+	sepname = separation_name(rs, dsc->page[i].label);
+	/* Check if this is a duplicate separation, in which case
+	 * we don't want to rename it twice.
+	 */
+	for (j=1; j<i; j++)
+	    if (strcmp(dsc->page[j].label, dsc->page[i].label) == 0) {
+		sepname = NULL;	/* don't do it */
+		duplicated = 1;
+	    }
+	/* If the separation name already exists in the renamed
+	 * list, don't rename it */ 
+	if (sepname) {
+	    for (j=1; j<i; j++) {
+	        if (strcmp(renamed[j], sepname) == 0) {
+		    sepname = NULL;	/* don't do it */
+		    duplicated = 1;
+		    break;
+		}
+	    }
+	}
+	if (sepname == NULL)
+	    sepname = dsc->page[i].label;
+	renamed[i] = sepname;
+    }
+    if (duplicated)
+	return -1;	/* clash in separation names */
+    return 0;
+}
+
 /* Find separation begin and end */
 static int
 dcs2_separation(CDSC *dsc, int pagenum, 
@@ -1256,7 +1329,7 @@ dcs2_separation(CDSC *dsc, int pagenum,
 
 
 static int 
-fix_custom(Doc *doc, char *buf, int buflen, RENAME_SEPARATION *rs)
+fix_custom(Doc *doc, char *buf, int buflen, const char *renamed[])
 {
     /* Create a new %%DocumentCustomColors:
      * containing only those separations that exist
@@ -1271,7 +1344,7 @@ fix_custom(Doc *doc, char *buf, int buflen, RENAME_SEPARATION *rs)
     if ((dsc == NULL) || !dsc->dcs2)
 	return 0;
     for (i=1; i<(int)dsc->page_count; i++) {
-	const char *sepname = separation_name(rs, dsc->page[i].label);
+	const char *sepname = renamed[i];
 	missing = dcs2_separation(dsc, i, &fname, &begin, &end);
 	if (!missing && !is_process_colour(sepname)) {
 	    n++;
@@ -1287,7 +1360,7 @@ fix_custom(Doc *doc, char *buf, int buflen, RENAME_SEPARATION *rs)
 }
 
 static int
-fix_process(Doc *doc, char *buf, int buflen, RENAME_SEPARATION *rs)
+fix_process(Doc *doc, char *buf, int buflen, const char *renamed[])
 {
     /* Create a new %%DocumentProcessColors:
      * containing only those separations that exist
@@ -1302,7 +1375,7 @@ fix_process(Doc *doc, char *buf, int buflen, RENAME_SEPARATION *rs)
     if ((dsc == NULL) || !dsc->dcs2)
 	return 0;
     for (i=1; i<(int)dsc->page_count; i++) {
-	const char *sepname = separation_name(rs, dsc->page[i].label);
+	const char *sepname = renamed[i];
 	missing = dcs2_separation(dsc, i, &fname, &begin, &end);
 	if (!missing && is_process_colour(sepname)) {
 	    n++;
@@ -1319,7 +1392,7 @@ fix_process(Doc *doc, char *buf, int buflen, RENAME_SEPARATION *rs)
  * count is the number of lines already written.
  */
 static int 
-write_cmyk_custom(GFile *gf, Doc *doc, RENAME_SEPARATION *rs, int count)
+write_cmyk_custom(GFile *gf, Doc *doc, const char *renamed[], int count)
 {
     /* Create a new %%CMYKCustomColor:
      * containing only those colours that exist
@@ -1336,7 +1409,7 @@ write_cmyk_custom(GFile *gf, Doc *doc, RENAME_SEPARATION *rs, int count)
 	return 0;
     buf[sizeof(buf)-1] = '\0';
     for (i=1; i<(int)dsc->page_count; i++) {
-	const char *sepname = separation_name(rs, dsc->page[i].label);
+	const char *sepname = renamed[i];
 	missing = dcs2_separation(dsc, i, &fname, &begin, &end);
 	if (!missing && !is_process_colour(sepname)) {
 	    /* find colour values */
@@ -1367,7 +1440,7 @@ write_cmyk_custom(GFile *gf, Doc *doc, RENAME_SEPARATION *rs, int count)
  * count is the number of lines already written.
  */
 static int 
-write_rgb_custom(GFile *gf, Doc *doc, RENAME_SEPARATION *rs, int count)
+write_rgb_custom(GFile *gf, Doc *doc, const char *renamed[], int count)
 {
     /* Create a new %%RGBCustomColor:
      * containing only those colours that exist
@@ -1384,7 +1457,7 @@ write_rgb_custom(GFile *gf, Doc *doc, RENAME_SEPARATION *rs, int count)
 	return 0;
     buf[sizeof(buf)-1] = '\0';
     for (i=1; i<(int)dsc->page_count; i++) {
-	const char *sepname = separation_name(rs, dsc->page[i].label);
+	const char *sepname = renamed[i];
 	missing = dcs2_separation(dsc, i, &fname, &begin, &end);
 	if (!missing && !is_process_colour(sepname)) {
 	    /* find colour values */
@@ -1417,7 +1490,7 @@ write_platefile_comments(Doc *doc, GFile *docfile,
     GFile *epsfile, LPCTSTR epsname,
     int offset, FILE_POS file_offset, 
     BOOL dcs2_multi, BOOL write_all, BOOL missing_separations,
-    RENAME_SEPARATION *rs)
+    const char *renamed[], BOOL some_renamed)
 {
     int i;
     CDSC *dsc = doc->dsc;
@@ -1433,22 +1506,33 @@ write_platefile_comments(Doc *doc, GFile *docfile,
 	/* First find length of separation */
 	int missing;
 	const char *fname = NULL;
-	const char *sepname = separation_name(rs, dsc->page[i].label);
+	const char *sepname = renamed[i];
 	missing = dcs2_separation(dsc, i, &fname, &begin, &end);
 	len = end - begin;
 	/* Now write out platefile */
-	if (missing && (missing_separations || (rs != NULL))) {
+	if (missing && (missing_separations || some_renamed)) {
 	    if (debug & DEBUG_GENERAL)
 		app_msgf(doc->app, 
 		    "Skipping missing separation page %d \042%s\042\n",
 		    i, dsc->page[i].label);
 	}
 	else if (dcs2_multi) {
+	    int j;
+	    int duplicate = 0;
 	    memset(platename, 0, sizeof(platename));
 	    cs_to_narrow(platename, sizeof(platename)-1, epsname, 
 		(int)cslen(epsname)+1);
 	    strncat(platename, ".", sizeof(platename) - strlen(platename));
 	    strncat(platename, sepname, sizeof(platename) - strlen(platename));
+	    for (j=1; j<i; j++) {
+		/* Check if separation name is a duplicate */ 
+		if (strcmp(sepname, renamed[j]) == 0)
+		    duplicate = 1;
+	    }
+	    if (duplicate) /* append page number to make the filename unique */
+		snprintf(platename+strlen(platename), 
+		    sizeof(platename) - strlen(platename),
+		    ".%d", i);
 	    snprintf(outbuf, sizeof(outbuf)-1,  
 		"%%%%PlateFile: (%s) EPS Local %s",
 		sepname, platename);
@@ -1502,7 +1586,7 @@ static FILE_POS
 write_singlefile_separations(Doc *doc, GFile *docfile, 
     GFile *epsfile, 
     BOOL dcs2_multi, BOOL write_all, BOOL missing_separations,
-    RENAME_SEPARATION *rs)
+    BOOL some_renamed)
 {
     int i;
     CDSC *dsc = doc->dsc;
@@ -1515,7 +1599,7 @@ write_singlefile_separations(Doc *doc, GFile *docfile,
 	    const char *fname = NULL;
 	    missing = dcs2_separation(dsc, i, &fname, &begin, &end);
 	    len = end - begin;
-	    if (missing && (missing_separations || (rs != NULL))) {
+	    if (missing && (missing_separations || some_renamed)) {
 		if (debug & DEBUG_GENERAL)
 		    app_msgf(doc->app, 
 			"Skipping missing separation page %d \042%s\042\n",
@@ -1551,7 +1635,8 @@ write_singlefile_separations(Doc *doc, GFile *docfile,
 
 /* Check that two DCS2 files can be combined */
 static int 
-check_dcs2_combine(Doc *doc1, Doc *doc2, int tolerance)
+check_dcs2_combine(Doc *doc1, Doc *doc2, 
+    const char **renamed1, const char **renamed2, int tolerance)
 {
     CDSC *dsc1;
     CDSC *dsc2;
@@ -1587,9 +1672,9 @@ check_dcs2_combine(Doc *doc1, Doc *doc2, int tolerance)
     }
     /* Check that separations don't conflict */
     for (i=1; i<(int)dsc1->page_count; i++) {
- 	sepname1 = dsc1->page[i].label;
+ 	sepname1 = renamed1[i];
 	for (j=1; i<(int)dsc2->page_count; i++) {
- 	    sepname2 = dsc2->page[j].label;
+ 	    sepname2 = renamed2[j];
 	    if (strcmp(sepname1, sepname2) == 0) {
 		app_msgf(doc1->app, 
 		    "Separation \042%s\042 appears in both files\n",
@@ -1642,12 +1727,39 @@ copy_dcs2(Doc *doc, GFile *docfile,
     BOOL found_endheader = FALSE;
     BOOL ignore_continuation = FALSE;
     int count;
+    const char **renamed1 = NULL;
+    const char **renamed2 = NULL;
     
     if (dsc->page_count == 0)
 	return -1;
 
-    if (doc2 && check_dcs2_combine(doc, doc2, tolerance))
+    /* Get renamed separations */
+    renamed1 = (const char **)malloc(sizeof(const char *) * dsc->page_count);
+    if (renamed1 == NULL)
 	return -1;
+    if (rename_separations(dsc, rs, renamed1) != 0) {
+	free((void *)renamed1);
+	return -1;
+    }
+    if (doc2) {
+	renamed2 = (const char **)
+	    malloc(sizeof(const char *) * doc2->dsc->page_count);
+	if (renamed2 == NULL) {
+	    free((void *)renamed1);
+	    return -1;
+	}
+	if (rename_separations(doc2->dsc, rs, renamed2) != 0) {
+	    free((void *)renamed1);
+	    free((void *)renamed2);
+	    return -1;
+	}
+    }
+
+    if (doc2 && check_dcs2_combine(doc, doc2, renamed1, renamed2, tolerance)) {
+	free((void *)renamed1);
+	free((void *)renamed2);
+	return -1;
+    }
 
     gfile_seek(docfile, dsc->page[0].begin, gfile_begin);
     memset(buf, 0, sizeof(buf)-1);
@@ -1722,43 +1834,51 @@ copy_dcs2(Doc *doc, GFile *docfile,
     if ((rs!=NULL) || missing_separations || doc2) {
 	int n;	/* number of entries written */
         strncpy(outbuf, process_str, sizeof(outbuf));
-	n = fix_process(doc, outbuf, sizeof(outbuf), rs);
+	n = fix_process(doc, outbuf, sizeof(outbuf), renamed1);
 	if (doc2)
-	    n += fix_process(doc2, outbuf, sizeof(outbuf), rs);
+	    n += fix_process(doc2, outbuf, sizeof(outbuf), renamed2);
 	if (n) {
 	    gfile_write(epsfile, outbuf, without_eol(outbuf, strlen(outbuf)));
 	    gfile_puts(epsfile, eol_str);	/* change EOL */
 	}
         strncpy(outbuf, custom_str, sizeof(outbuf));
-	n = fix_custom(doc, outbuf, sizeof(outbuf), rs);
+	n = fix_custom(doc, outbuf, sizeof(outbuf), renamed1);
 	if (doc2)
-	    n += fix_custom(doc2, outbuf, sizeof(outbuf), rs);
+	    n += fix_custom(doc2, outbuf, sizeof(outbuf), renamed2);
 	if (n) {
 	    gfile_write(epsfile, outbuf, without_eol(outbuf, strlen(outbuf)));
 	    gfile_puts(epsfile, eol_str);	/* change EOL */
 	}
-	n = write_cmyk_custom(epsfile, doc, rs, 0);
+	n = write_cmyk_custom(epsfile, doc, renamed1, 0);
 	if (doc2)
-	    write_cmyk_custom(epsfile, doc2, rs, n);
-	n = write_rgb_custom(epsfile, doc, rs, 0);
+	    write_cmyk_custom(epsfile, doc2, renamed2, n);
+	n = write_rgb_custom(epsfile, doc, renamed1, 0);
 	if (doc2)
-	    write_rgb_custom(epsfile, doc2, rs, n);
+	    write_rgb_custom(epsfile, doc2, renamed2, n);
     }
 
 
     /* Write out the platefile comments and any multiple file separations */
     len = write_platefile_comments(doc, docfile, epsfile, epsname, 
 	offset, file_offset, 
-	dcs2_multi, write_all, missing_separations, rs);
-    if (len == (FILE_POS)-1)
+	dcs2_multi, write_all, missing_separations, renamed1, (rs != NULL));
+    if (len == (FILE_POS)-1) {
+	free((void *)renamed1);
+	if (renamed2)
+	    free((void *)renamed2);
 	return -1;
+    }
     file_offset += len;
     if (doc2) {
 	len = write_platefile_comments(doc2, docfile2, epsfile, epsname, 
 	    offset, file_offset, 
-	    dcs2_multi, write_all, missing_separations, rs);
-	if (len == (FILE_POS)-1)
+	    dcs2_multi, write_all, missing_separations, renamed2, (rs != NULL));
+	if (len == (FILE_POS)-1) {
+	    free((void *)renamed1);
+	    if (renamed2)
+		free((void *)renamed2);
 	    return -1;
+	}
 	file_offset += len;
     }
 
@@ -1795,10 +1915,13 @@ copy_dcs2(Doc *doc, GFile *docfile,
 
     /* Write out single file separations */
     write_singlefile_separations(doc, docfile, epsfile, 
-        dcs2_multi, write_all, missing_separations, rs);
+        dcs2_multi, write_all, missing_separations, (rs != NULL));
     if (doc2)
         write_singlefile_separations(doc2, docfile2, epsfile, 
-            dcs2_multi, write_all, missing_separations, rs);
+            dcs2_multi, write_all, missing_separations, (rs != NULL));
+    free((void *)renamed1);
+    if (renamed2)
+	free((void *)renamed2);
     return 0;
 }
 
@@ -2080,7 +2203,7 @@ make_eps_pict(Doc *doc, IMAGE *img, CDSCBBOX *bbox,
 	((epsname[len-1] == 's') || (epsname[len-1] == 'S')) )
 	type = CMAC_TYPE_SINGLE;
     else if ((len > 5) &&
-	(epsname[len-5] == '.') &&
+	((epsname[len-5] == '.') || (epsname[len-5] == '/')) &&
 	((epsname[len-4] == 'r') || (epsname[len-4] == 'R')) &&
 	((epsname[len-3] == 's') || (epsname[len-3] == 'S')) &&
 	((epsname[len-2] == 'r') || (epsname[len-2] == 'R')) &&
