@@ -13,12 +13,13 @@
   Fax +61 3 9886 6616.
 */
 
-/* $Id: cimg.c,v 1.18 2004/01/16 12:03:06 ghostgum Exp $ */
+/* $Id: cimg.c,v 1.19 2004/07/08 09:13:54 ghostgum Exp $ */
 /* Common image format and conversion functions */
 
 #include "common.h"
 #include "gdevdsp.h"
 #include "cimg.h"
+#include "clzw.h"
 
 static int a85write(char *line, unsigned char *buf, int count);
 
@@ -1798,7 +1799,7 @@ packbits(BYTE *comp, BYTE *raw, int length)
  */
 int 
 image_to_eps(GFile *f, IMAGE *img, int llx, int lly, int urx, int ury,
-    float fllx, float flly, float furx, float fury, int use_a85, int use_pack)
+    float fllx, float flly, float furx, float fury, int use_a85, int compress)
 {
     int x, y;
     int topfirst;
@@ -1813,6 +1814,7 @@ image_to_eps(GFile *f, IMAGE *img, int llx, int lly, int urx, int ury,
     unsigned char *packout;
     int packin_count;
     int packout_count;
+    int packout_len;
     int separate = 0;		/* Write components separately */
     int depth;
     int compwidth;		/* width of one row of one component in bytes */
@@ -1820,6 +1822,8 @@ image_to_eps(GFile *f, IMAGE *img, int llx, int lly, int urx, int ury,
     BOOL invert = FALSE;	/* black=0 for FALSE, black=1 for TRUE */
     unsigned char *convert_row = NULL;
     char buf[MAXSTR];
+    lzw_state_t *lzw = NULL;
+#define USE_LZW 2
 
     if ((fllx >= furx) || (flly >= fury))
 	hires_bbox_valid = 0;
@@ -1873,7 +1877,8 @@ image_to_eps(GFile *f, IMAGE *img, int llx, int lly, int urx, int ury,
     packin = (unsigned char *)malloc(compwidth * ncomp);
     if (packin == NULL)
 	return -1;
-    packout = (unsigned char *)malloc(compwidth * ncomp * 5 / 4 + 4);
+    packout_len = compwidth * ncomp * 5 / 4 + 4;
+    packout = (unsigned char *)malloc(packout_len);
     if (packout == NULL) {
 	free(packin);
 	return -1;
@@ -1886,6 +1891,16 @@ image_to_eps(GFile *f, IMAGE *img, int llx, int lly, int urx, int ury,
 	    return -1;
 	}
     }
+    if (compress == USE_LZW) {
+	lzw = lzw_new();
+	if (lzw == (lzw_state_t *)NULL) {
+	    free(packin);
+	    free(packout);
+ 	    if (convert_row)
+	        free(convert_row);
+	    return -1;
+	}
+    }
 
     gfile_puts(f, "%%!PS-Adobe-3.0 EPSF-3.0\n");
     snprintf(buf, sizeof(buf)-1, 
@@ -1918,7 +1933,7 @@ image_to_eps(GFile *f, IMAGE *img, int llx, int lly, int urx, int ury,
         gfile_puts(f, "/DeviceRGB setcolorspace\n");
     else if (ncomp == 4)
         gfile_puts(f, "/DeviceCMYK setcolorspace\n");
-    if ((ncomp > 1) && use_pack) {
+    if ((ncomp > 1) && compress) {
 	/* Include as RRRRGGGGBBBB not RGBRGBRGBRGB
 	 * (or CCCCMMMMYYYYKKKK not CMYKCMYKCMYKCMYK)
 	 * Since this compresses with RLE better.
@@ -1932,7 +1947,9 @@ image_to_eps(GFile *f, IMAGE *img, int llx, int lly, int urx, int ury,
 	    gfile_puts(f, "/infile currentfile /ASCII85Decode filter");
 	else
 	    gfile_puts(f, "/infile currentfile /ASCIIHexDecode filter");
-	if (use_pack)
+	if (compress == USE_LZW)
+	    gfile_puts(f, " /LZWDecode filter");
+	else if (compress)
 	    gfile_puts(f, " /RunLengthDecode filter");
 	gfile_puts(f, " def\n");
 	for (i=0; i<ncomp; i++) {
@@ -1974,7 +1991,9 @@ image_to_eps(GFile *f, IMAGE *img, int llx, int lly, int urx, int ury,
 	    gfile_puts(f, " /DataSource currentfile /ASCII85Decode filter\n");
 	else
 	    gfile_puts(f, " /DataSource currentfile /ASCIIHexDecode filter\n");
-	if (use_pack)
+	if (compress == USE_LZW)
+	    gfile_puts(f, " /LZWDecode filter\n");
+	else if (compress)
 	    gfile_puts(f, " /RunLengthDecode filter\n");
     }
     gfile_puts(f, ">>\nimage\n");
@@ -2012,9 +2031,25 @@ image_to_eps(GFile *f, IMAGE *img, int llx, int lly, int urx, int ury,
 		}
 	    }
 	}
-	if (use_pack)
+	if (compress == USE_LZW) {
+	    int inlen = packin_count;
+	    int outlen = packout_len - packout_count;
+	    lzw_compress(lzw, packin, &inlen, packout+packout_count, &outlen);
+	    packout_count += outlen;
+	    if (y == (int)img->height-1) {
+		/* This is the last row */
+		/* Flush and EOD */
+		inlen = 0;	/* EOD */
+		outlen = packout_len - packout_count;
+		lzw_compress(lzw, packin, &inlen, packout+packout_count, 
+		    &outlen);
+		packout_count += outlen;
+	    }
+	}
+	else if (compress) {
 	    packout_count += 
 		packbits(packout+packout_count, packin, packin_count);
+	}
 	else {
 	    memcpy(packout+packout_count, packin, packin_count);
 	    packout_count += packin_count;
@@ -2058,11 +2093,14 @@ image_to_eps(GFile *f, IMAGE *img, int llx, int lly, int urx, int ury,
     gfile_puts(f, "showpage\n");
     gfile_puts(f, "%%Trailer\n");
     gfile_puts(f, "%%EOF\n");
+    if (lzw)
+	lzw_free(lzw);
     if (convert)
 	free(convert_row);
     free(packin);
     free(packout);
     return 0;
+#undef USE_LZW
 }
 
 int 
