@@ -13,7 +13,7 @@
   Fax +61 3 9886 6616.
 */
 
-/* $Id: cmac.c,v 1.7 2004/01/16 12:03:06 ghostgum Exp $ */
+/* $Id: cmac.c,v 1.8 2004/09/04 00:41:39 ghostgum Exp $ */
 /* Macintosh AppleSingle, AppleDouble and MacBinary file formats */
 /* Macintosh does not use a flat file system.
  * Each file can have a data fork and a resource fork.
@@ -116,14 +116,14 @@ CMACFILE *get_mactype(GFile *f)
 		(data[0]==0x0) && 
 		(data[1] >= 1) && (data[1] <= 63) && 
 		(data[74]==0x0) && (data[82]==0x0) &&
-		(data[65]>='0') && (data[65]<='z') &&
-		(data[66]>='0') && (data[66]<='z') &&
-		(data[67]>='0') && (data[67]<='z') &&
-		(data[68]>='0') && (data[68]<='z') &&
-		(data[69]>='0') && (data[69]<='z') &&
-		(data[70]>='0') && (data[70]<='z') &&
-		(data[71]>='0') && (data[71]<='z') &&
-		(data[72]>='0') && (data[72]<='z')) {
+		(data[65]>=' ') && (data[65]<='z') &&
+		(data[66]>=' ') && (data[66]<='z') &&
+		(data[67]>=' ') && (data[67]<='z') &&
+		(data[68]>=' ') && (data[68]<='z') &&
+		(data[69]>=' ') && (data[69]<='z') &&
+		(data[70]>=' ') && (data[70]<='z') &&
+		(data[71]>=' ') && (data[71]<='z') &&
+		(data[72]>=' ') && (data[72]<='z')) {
 		type = CMAC_TYPE_MACBIN;
 	    }
 	    else {
@@ -825,7 +825,8 @@ static int dump_macfile(LPCTSTR filename)
 void usage(void)
 {
     fprintf(stdout, "Usage:\n\
-    Dump file: cmac -d input.eps\n\
+    File info: cmac -i input.eps\n\
+    Extract data fork: cmac -d input output\n\
     Extract PICT: cmac -x input.eps output.pict\n\
     Write AppleSingle: cmac -1 input.eps input.pict output.eps\n\
     Write AppleDouble: cmac -2 input.pict ._output.eps\n\
@@ -848,13 +849,39 @@ int main(int argc, char *argv[])
 	return 1;
     }
     switch (argv[1][1]) {
-	case 'd':
+	case 'i':
 	    if (argc != 3) {
 		usage();
 		return 1;
 	    }
 	    dump_macfile(argv[2]);
 	    break;
+	case 'd':
+	    /* Extract data fork */
+	    if (argc != 4) {
+		usage();
+		return 1;
+	    }
+	    f = gfile_open(argv[2], gfile_modeRead);
+	    if (f == NULL) {
+		fprintf(stdout, "Failed to open file \042%s\042\n", argv[1]);
+		return -1;
+	    }
+	    mac = get_mactype(f);
+	    if (mac == NULL) {
+		fprintf(stdout, "Not a Mac file with resource fork\n");
+		code = 1;
+	    }
+	    if (code == 0) {
+		code = extract_mac_data(f, argv[3], 
+		    mac->data_begin, mac->data_length, 0);
+		if (code)
+		    fprintf(stdout, "Failed to extract data fork.\n");
+		else
+		    fprintf(stdout, "Success\n");
+	    }
+	    gfile_close(f);
+	    break;
 	case 'x':
 	    /* Extract PICT preview */
 	    if (argc != 4) {
