@@ -15,7 +15,7 @@
   the copyright notice and this notice be preserved on all copies.
 */
 
-/* $Id: dscparse.h,v 1.16 2004/01/16 08:55:37 ghostgum Exp $ */
+/* $Id: dscparse.h,v 1.17 2004/07/08 09:01:30 ghostgum Exp $ */
 /* Interface for the DSC parser. */
 
 #ifndef dscparse_INCLUDED
@@ -501,6 +501,8 @@ char dummy[1024];
     /* public data */
     /* Added 2003-07-15 */
     CDSCMACBIN *macbin;		/* Mac Binary header */
+    /* Added 2004-04-28 */
+    GSBOOL dcs1;		/* True if dcs2 set but really DCS 1.0 */
 };
 
 
