@@ -1,4 +1,4 @@
-/* Copyright (C) 1995-2004 Ghostgum Software Pty Ltd.  All rights reserved.
+/* Copyright (C) 1995-2005 Ghostgum Software Pty Ltd.  All rights reserved.
 
   This software is provided AS-IS with no warranty, either express or
   implied.
@@ -13,7 +13,7 @@
   Fax +61 3 9886 6616.
 */
 
-/* $Id: epstool.c,v 1.40 2004/01/16 12:03:06 ghostgum Exp $ */
+/* $Id: epstool.c,v 1.57 2005/01/07 11:27:37 ghostgum Exp $ */
 
 #include "common.h"
 #include "dscparse.h"
@@ -44,9 +44,9 @@
 #endif
 
 const char *epstool_name = "epstool";
-const char *epstool_version = "3.03";	 /* should be EPSTOOL_VERSION */
-const char *epstool_date = "2003-10-18"; /* should be EPSTOOL_DATE */
-const char *copyright = "Copyright 1995-2004 Ghostgum Software Pty Ltd";
+const char *epstool_version = "3.07";	 /* should be EPSTOOL_VERSION */
+const char *epstool_date = "2005-01-07"; /* should be EPSTOOL_DATE */
+const char *copyright = "Copyright 1995-2005 Ghostgum Software Pty Ltd";
 
 const char *cmd_help = "\
 Commands (one only):\n\
@@ -89,6 +89,7 @@ Options:\n\
   --mac-single\n\
   --missing-separations\n\
   --output filename\n\
+  --page-number\n\
   --quiet\n\
   --rename-separation old_name new_name\n\
   --replace-composite\n\
@@ -142,7 +143,7 @@ typedef struct OPT_s {
     BOOL bbox;			/* --bbox */
     int dscwarn;		/* --ignore-warnings or --ignore-errors */
     TCHAR gs[MAXSTR];		/* --gs command */
-    TCHAR gsargs[MAXSTR];	/* --gs-args arguments */
+    TCHAR gsargs[MAXSTR*4];	/* --gs-args arguments */
     TCHAR input[MAXSTR];	/* filename */
     TCHAR output[MAXSTR];	/* --output filename or (second) filename */
     TCHAR user_preview[MAXSTR];	/* --add-user-preview filename */
@@ -160,6 +161,7 @@ typedef struct OPT_s {
     RENAME_SEPARATION *rename_sep; /* --rename-separation */
     CMAC_TYPE mac_type;		/* --mac-binary, --mac-double, --mac-single */
 				/* or --mac-rsrc */
+    int page;			/* --page-number for --bitmap */
 } OPT;
 
 
@@ -409,6 +411,16 @@ parse_args(OPT *opt, int argc, TCHAR *argv[])
 		return arg;
 	    csncpy(opt->device, argv[arg], sizeof(opt->device)/sizeof(TCHAR)-1);
 	}
+	else if (cscmp(p, TEXT("--page-number")) == 0) {
+	    char buf[MAXSTR];
+	    arg++;
+	    if (arg == argc)
+		return arg;
+	    cs_to_narrow(buf, sizeof(buf)-1, argv[arg], cslen(argv[arg])+1);
+	    opt->page = atoi(buf) - 1;
+	    if (opt->page < 0)
+		opt->page = 0;
+	}
 	else if ((cscmp(p, TEXT("--bbox")) == 0) ||
 	    (cscmp(p, TEXT("-b")) == 0)) {
 	    opt->bbox = TRUE;
@@ -528,7 +540,8 @@ epstool_open_document(GSview *app, OPT *opt, TCHAR *name)
 	code = -1;
     }
 
-    if ((code == 0) && (!doc->dsc->epsf) && (opt->cmd != CMD_DCS2_REPORT)) {
+    if ((code == 0) && (opt->cmd != CMD_DCS2_REPORT) && 
+	   ((doc->dsc == NULL) || (!doc->dsc->epsf)) ) {
 	debug |= DEBUG_LOG;
 	app_csmsgf(app, 
 	    TEXT("Input file \042%s\042 is not EPSF.\n"), name);
@@ -772,6 +785,8 @@ epstool_add_preview(Doc *doc, OPT *opt)
     IMAGE *img = NULL;
     if (opt->device[0] != '\0')
 	device = opt->device;
+    else if (opt->cmd == CMD_INTERCHANGE)
+	device = MONO_DEVICE;
     if (opt->cmd == CMD_TIFF4)
 	device = MONO_DEVICE;
 
@@ -905,6 +920,7 @@ epstool_bitmap(Doc *doc, OPT *opt)
     CDSCBBOX bbox;
     CDSCFBBOX hires_bbox;
     LPCTSTR device = COLOUR_DEVICE;
+    int page = opt->page;
     if (opt->device[0] != '\0')
 	device = opt->device;
 
@@ -924,7 +940,12 @@ epstool_bitmap(Doc *doc, OPT *opt)
 	hires_bbox.fury = (float)bbox.ury;
     }
 
-    code = make_preview_file(doc, opt, 0, opt->output,
+    if (doc->dsc->page_count == 0)
+	page = 0;
+    else if ((opt->page < 0) || (opt->page >= (int)doc->dsc->page_count))
+	return -1;	/* invalid page */
+
+    code = make_preview_file(doc, opt, page, opt->output,
 	device, opt->dpi, &bbox, &hires_bbox, opt->bbox);
 
     return code;
@@ -1092,7 +1113,7 @@ epstool_dcs2_report(Doc *doc)
     const char *type = "Unknown";
     const char *preview = "Unknown";
 
-    if (dsc->dcs2) {
+    if (dsc && dsc->dcs2) {
 	DSC_OFFSET length = 0;
 	GFile *f;
 	/* Check for missing composite */
@@ -1118,10 +1139,18 @@ epstool_dcs2_report(Doc *doc)
     }
 
     /* Document type */
-    if (dsc == NULL)
-	type = "Unknown";
-    else if (dsc->dcs2)
-	type = "DCS2.0";
+    if (dsc == NULL) {
+	if (doc->doctype == DOC_PDF)
+	    type = "PDF";
+	else
+	    type = "Unknown";
+    }
+    else if (dsc->dcs2) {
+	if (dsc->dcs1)
+	    type = "DCS1.0";
+	else
+	    type = "DCS2.0";
+    }
     else if (dsc->epsf)
 	type = "EPSF";
     else if (dsc->pdf)
@@ -1129,6 +1158,8 @@ epstool_dcs2_report(Doc *doc)
     else
 	type = "DSC";
     fprintf(stdout, "Type\t%s\n", type);
+    if (dsc == NULL)
+	return 1;
 
     /* Preview type */
     switch (dsc->preview) {
@@ -1198,6 +1229,7 @@ epstool_dcs2_check_files(Doc *doc, OPT *opt)
     CDSC *dsc = doc->dsc;
     GFile *f;
     const char *fname;
+    const char **renamed1 = NULL;
     for (i=1; (i<(int)dsc->page_count); i++) {
 	/* First find length of separation */
 	fname = dsc_find_platefile(dsc, i);
@@ -1228,6 +1260,17 @@ epstool_dcs2_check_files(Doc *doc, OPT *opt)
 	    }
 	}
     }
+    /* Test if duplicate separations would occur */
+    renamed1 = (const char **)malloc(sizeof(const char *) * dsc->page_count);
+    if (renamed1 != NULL) {
+	if (rename_separations(dsc, opt->rename_sep, renamed1) != 0) {
+	    code = -1;
+	    if (!opt->quiet)
+	      app_msgf(doc->app, "Duplicate separations are not permitted.\n");
+	}
+	free((void *)renamed1);
+    }
+
     return code;
 }
 
@@ -1398,13 +1441,14 @@ calculate_bbox(Doc *doc, OPT *opt, LPCTSTR psname, CDSCBBOX *bbox, CDSCFBBOX *hi
 {
     FILE *bboxfile;
     TCHAR bboxname[MAXSTR];
-    TCHAR command[MAXSTR*4];
+    TCHAR command[MAXSTR*8];
     char line[MAXSTR];
     int got_bbox = 0;
     int got_hires_bbox = 0;
     int code = 0;
     int llx, lly, urx, ury;
     float fllx, flly, furx, fury;
+    const int pagesize = 9400; /* Must be < 9419 on 7.07, < 150976 on 8.x */
     const int offset = 3000;
     if ((bboxfile = app_temp_file(doc->app, bboxname, 
 	sizeof(bboxname)/sizeof(TCHAR))) == (FILE *)NULL) {
@@ -1415,10 +1459,10 @@ calculate_bbox(Doc *doc, OPT *opt, LPCTSTR psname, CDSCBBOX *bbox, CDSCFBBOX *hi
     fclose(bboxfile);
 
     csnprintf(command, sizeof(command)/sizeof(TCHAR), TEXT(
-	"\042%s\042 %s %s -dNOPAUSE -dBATCH -sDEVICE=bbox \
- -c \042<</PageOffset [%d %d]>> setpagedevice\042 -f \042%s\042"), 
-	opt->gs, opt->gsargs, opt->quiet ? TEXT("-dQUIET") : TEXT(""), 
-	offset, offset, psname);
+	"\042%s\042 %s -dNOPAUSE -dBATCH -sDEVICE=bbox %s \
+ -c \042<</PageSize [%d %d] /PageOffset [%d %d]>> setpagedevice\042 -f \042%s\042"), 
+	opt->gs, opt->quiet ? TEXT("-dQUIET") : TEXT(""), opt->gsargs,
+	pagesize, pagesize, offset, offset, psname);
     if (!opt->quiet)
 	app_csmsgf(doc->app, TEXT("%s\n"), command);
     code = exec_program(command, -1, fileno(stdout), -1, NULL, NULL, bboxname);
@@ -1529,7 +1573,7 @@ make_preview_file(Doc *doc, OPT *opt, int page,
     GFile *f;
     int code = 0;
     TCHAR tpsname[MAXSTR];
-    TCHAR command[MAXSTR*4];
+    TCHAR command[MAXSTR*8];
     int width, height;
     float xoffset, yoffset;
 
@@ -1566,10 +1610,10 @@ make_preview_file(Doc *doc, OPT *opt, int page,
 	
     /* Make the preview image */
     csnprintf(command, sizeof(command)/sizeof(TCHAR),
-	TEXT("\042%s\042 %s %s -dNOPAUSE -dBATCH -sDEVICE=%s -sOutputFile=\042%s\042 -r%g -g%dx%d -c %f %f translate -f \042%s\042"), 
-	opt->gs, opt->gsargs, opt->quiet ? TEXT("-dQUIET") : TEXT(""), 
-	device, preview, dpi, width, height, 
-	xoffset, yoffset, tpsname);
+	TEXT("\042%s\042 %s -dNOPAUSE -dBATCH -sDEVICE=%s -sOutputFile=\042%s\042 -r%g -g%dx%d %s -c %f %f translate -f \042%s\042"), 
+	opt->gs, opt->quiet ? TEXT("-dQUIET") : TEXT(""), 
+	device, (preview[0]=='\0' ? TEXT("-") : preview), dpi, width, height, 
+	opt->gsargs, xoffset, yoffset, tpsname);
     if (!opt->quiet)
 	app_csmsgf(doc->app, TEXT("%s\n"), command);
     code = exec_program(command, -1, fileno(stdout), fileno(stderr),
@@ -1797,7 +1841,7 @@ epstool_dcs2_composite(Doc *doc, OPT *opt, GFile *compfile)
 		hires_bbox.fllx, hires_bbox.flly, 
 		hires_bbox.furx, hires_bbox.fury, 
 		TRUE /* use ASCII85Decode */, 
-		TRUE /* use RunLengthDecode */ );
+		2 /* use LZWDecode */ );
     }
 
     free(img.image);
